#include <pybind11/pybind11.h>

#include "CoreShell.cpp"

PYBIND11_MODULE(CoreShellInterface, module)
{
    module.doc() = "Lorenz-Mie Theory (LMT) C++ binding module for PyMieSim Python package.";


     pybind11::class_<CORESHELL::State>(module, "CppCoreShellState");


      pybind11::class_<CORESHELL::Scatterer>(module, "CORESHELL")
      .def(pybind11::init<double &, double &, double &, double &, complex128 &, complex128 &, double &, CVector &>(),
           pybind11::arg("Wavelength"),
           pybind11::arg("Amplitude"),
           pybind11::arg("CoreDiameter"),
           pybind11::arg("ShellDiameter"),
           pybind11::arg("CoreIndex"),
           pybind11::arg("ShellIndex"),
           pybind11::arg("nMedium"),
           pybind11::arg("Jones")
          )

       .def("S1S2",
            &CORESHELL::Scatterer::PyS1S2_,
            pybind11::arg("Phi") )

      .def("Fields",
           &CORESHELL::Scatterer::PyFields_,
           pybind11::arg("Phi"),
           pybind11::arg("Theta"),
           pybind11::arg("R") )

      .def("FullFields",
           &CORESHELL::Scatterer::PyFullFields_,
           pybind11::arg("Sampling"),
           pybind11::arg("R") )

      .def("an", pybind11::overload_cast<>(&CORESHELL::Scatterer::GetPyAn))
      .def("bn", pybind11::overload_cast<>(&CORESHELL::Scatterer::GetPyBn))

      .def_property_readonly("Qsca",  &CORESHELL::Scatterer::GetQsca)
      .def_property_readonly("Qext",  &CORESHELL::Scatterer::GetQext)
      .def_property_readonly("Qabs",  &CORESHELL::Scatterer::GetQabs)
      .def_property_readonly("Qback", &CORESHELL::Scatterer::GetQback)
      .def_property_readonly("Qpr",   &CORESHELL::Scatterer::GetQpr)

      .def_property_readonly("Csca",  &CORESHELL::Scatterer::GetCsca)
      .def_property_readonly("Cext",  &CORESHELL::Scatterer::GetCext)
      .def_property_readonly("Cabs",  &CORESHELL::Scatterer::GetCabs)
      .def_property_readonly("Cback", &CORESHELL::Scatterer::GetCback)
      .def_property_readonly("Cpr",   &CORESHELL::Scatterer::GetCpr)

      .def_property_readonly("g",     &CORESHELL::Scatterer::GetG)

      .def_readwrite("State", &CORESHELL::Scatterer::state)
      .def_readwrite("Area", &CORESHELL::Scatterer::Area)
      .def_readwrite("SizeParameter", &CORESHELL::Scatterer::SizeParameter)
      ;
}







// -
