#include <pybind11/pybind11.h>

#include "Cylinder.cpp"

PYBIND11_MODULE(CylinderInterface, module)
{
    module.doc() = "Lorenz-Mie Theory (LMT) C++ binding module for PyMieSim Python package.";


     pybind11::class_<CYLINDER::State>(module, "CppCylinderState");


      pybind11::class_<CYLINDER::Scatterer>(module, "CYLINDER")
      .def(pybind11::init<double&, double&, double&, complex128&, double&, CVector&>(),
           pybind11::arg("Wavelength"),
           pybind11::arg("Amplitude"),
           pybind11::arg("Diameter"),
           pybind11::arg("Index"),
           pybind11::arg("nMedium"),
           pybind11::arg("Jones")
          )

       .def("S1S2",
            &CYLINDER::Scatterer::PyS1S2_,
            pybind11::arg("Phi") )

      .def("Fields",
           &CYLINDER::Scatterer::PyFields_,
           pybind11::arg("Phi"),
           pybind11::arg("Theta"),
           pybind11::arg("R") )

      .def("FullFields",
           &CYLINDER::Scatterer::PyFullFields_,
           pybind11::arg("Sampling"),
           pybind11::arg("R") )

      .def("a1n", pybind11::overload_cast<>(&CYLINDER::Scatterer::GetPyA1n))
      .def("b1n", pybind11::overload_cast<>(&CYLINDER::Scatterer::GetPyB1n))
      .def("a2n", pybind11::overload_cast<>(&CYLINDER::Scatterer::GetPyA2n))
      .def("b2n", pybind11::overload_cast<>(&CYLINDER::Scatterer::GetPyB2n))

      .def_property_readonly("Qsca",  &CYLINDER::Scatterer::GetQsca)
      .def_property_readonly("Qext",  &CYLINDER::Scatterer::GetQext)
      .def_property_readonly("Qabs",  &CYLINDER::Scatterer::GetQabs)
      .def_property_readonly("Qback", &CYLINDER::Scatterer::GetQback)
      .def_property_readonly("Qpr",   &CYLINDER::Scatterer::GetQpr)

      .def_property_readonly("Csca",  &CYLINDER::Scatterer::GetCsca)
      .def_property_readonly("Cext",  &CYLINDER::Scatterer::GetCext)
      .def_property_readonly("Cabs",  &CYLINDER::Scatterer::GetCabs)
      .def_property_readonly("Cback", &CYLINDER::Scatterer::GetCback)
      .def_property_readonly("Cpr",   &CYLINDER::Scatterer::GetCpr)

      .def_property_readonly("g",     &CYLINDER::Scatterer::GetG)

      .def_readwrite("State", &CYLINDER::Scatterer::state)
      .def_readwrite("Area", &CYLINDER::Scatterer::Area)
      .def_readwrite("SizeParameter", &CYLINDER::Scatterer::SizeParameter)
      ;
}







// -
