#include <pybind11/pybind11.h>

#include "Sphere.cpp"

PYBIND11_MODULE(SphereInterface, module)
{
    module.doc() = "Lorenz-Mie Theory (LMT) C++ binding module for PyMieSim Python package.";


      pybind11::class_<SPHERE::State>(module, "CppSphereState");


      pybind11::class_<SPHERE::Scatterer>(module, "SPHERE")
      .def(pybind11::init<double&, double&, double&, complex128&, double&, CVector&>(),
           pybind11::arg("Wavelength"),
           pybind11::arg("Amplitude"),
           pybind11::arg("Diameter"),
           pybind11::arg("Index"),
           pybind11::arg("nMedium"),
           pybind11::arg("Jones")
          )

      .def(pybind11::init<>())

      .def("S1S2",
      &SPHERE::Scatterer::PyS1S2_,
      pybind11::arg("Phi") )

      .def("Fields",
      &SPHERE::Scatterer::PyFields_,
      pybind11::arg("Phi"),
      pybind11::arg("Theta"),
      pybind11::arg("R") )

      .def("FullFields",
      &SPHERE::Scatterer::PyFullFields_,
      pybind11::arg("Sampling"),
      pybind11::arg("R") )


      .def("an", pybind11::overload_cast<>(&SPHERE::Scatterer::GetPyAn))
      .def("bn", pybind11::overload_cast<>(&SPHERE::Scatterer::GetPyBn))
      .def("cn", pybind11::overload_cast<>(&SPHERE::Scatterer::GetPyCn))
      .def("dn", pybind11::overload_cast<>(&SPHERE::Scatterer::GetPyDn))

      .def_property_readonly("Qsca",  &SPHERE::Scatterer::GetQsca)
      .def_property_readonly("Qext",  &SPHERE::Scatterer::GetQext)
      .def_property_readonly("Qabs",  &SPHERE::Scatterer::GetQabs)
      .def_property_readonly("Qback", &SPHERE::Scatterer::GetQback)
      .def_property_readonly("Qpr",   &SPHERE::Scatterer::GetQpr)


      .def_property_readonly("Csca",  &SPHERE::Scatterer::GetCsca)
      .def_property_readonly("Cext",  &SPHERE::Scatterer::GetCext)
      .def_property_readonly("Cabs",  &SPHERE::Scatterer::GetCabs)
      .def_property_readonly("Cback", &SPHERE::Scatterer::GetCback)
      .def_property_readonly("Cpr",   &SPHERE::Scatterer::GetCpr)

      .def_property_readonly("g", &SPHERE::Scatterer::GetG)


      .def_readwrite("State", &SPHERE::Scatterer::state)
      .def_readwrite("Area", &SPHERE::Scatterer::Area)
      .def_readwrite("SizeParameter", &SPHERE::Scatterer::SizeParameter)
      ;
}







// -
